/*
 * Created on 08-Jun-2004
 * Created by Paul Gardner
 * Copyright (C) Azureus Software, Inc, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package com.biglybt.pif.ui.config;

/**
 * @author parg
 *
 * @see com.biglybt.pif.ui.model.BasicPluginConfigModel#addDirectoryParameter2(String, String, String)
 */
public interface
DirectoryParameter
	extends Parameter, ParameterWithHint
{
	public void
	setValue(
		String		str );

	public String
	getValue();

	/**
	 * Title of the dialog box shown when user clicks the browse button
	 *
	 * @since BiglyBT 1.9.0.1
	 */
	void setDialogTitleKey(String key);

	/**
	 * Message for the dialog box shown when user clicks the browse button
	 *
	 * @note: disabled because Windows doesn't show a message anymore :(
	 * @since BiglyBT 1.9.0.1
	 */
	//void setDialogMessageKey(String key);
}
