/*
 * Copyright (C) Azureus Software, Inc, All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.biglybt.core.peermanager.control;

import com.biglybt.core.config.COConfigurationManager;

public interface
PeerControlScheduler
{
	public static final int SCHEDULE_PERIOD_MILLIS = COConfigurationManager.getIntParameter( "peermanager.schedule.time" );
	public static final int SCHEDULE_PERIOD_MAX_CATCHUP = SCHEDULE_PERIOD_MILLIS>>2;

	public void
	register(
		PeerControlInstance	instance );

	public void
	unregister(
		PeerControlInstance	instance );

	public void
	updateScheduleOrdering();

	public SpeedTokenDispenser
	getSpeedTokenDispenser();

	/**
	 *
	 * @param override, set to true to disable weighted priorities, false to use user-configureable default
	 */
	public void overrideWeightedPriorities(boolean override);
	
	public int[]
	getPeerCount();
	
	public int[]
	getPieceCount();
}
